/*
   Copyright 2012-2023 Michael Pozhidaev <msp@luwrain.org>

   This file is part of LUWRAIN.

   LUWRAIN is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.

   LUWRAIN is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
*/

package org.luwrain.app.vk2;

import java.util.*;
import java.util.concurrent.atomic.*;
import java.io.*;

import com.vk.api.sdk.objects.messages.ConversationWithMessage;
import com.vk.api.sdk.oneofs.NewsfeedNewsfeedItemOneOf;

import org.luwrain.core.*;
import org.luwrain.core.events.*;
import org.luwrain.core.queries.*;
import org.luwrain.controls.*;
import org.luwrain.script.*;
import org.luwrain.app.base.*;
import org.luwrain.nlp.*;

import static org.luwrain.controls.ListUtils.*;

final class MainLayout extends LayoutBase
{
    private final App app;
        final ListArea<NewsfeedNewsfeedItemOneOf> newsArea;
    final ListArea<ConversationWithMessage> chatsArea;
    //    final ListArea requestsArea;

    MainLayout(App app)
    {
	super(app);
	this.app = app;
		this.newsArea = new ListArea<NewsfeedNewsfeedItemOneOf>(listParams((params)->{
			    params.name = "Новости";//FIXME:
			    params.model = new ListModel<>(app.news);
			    params.appearance = new NewsAppearance(app);
			    params.clickHandler = this::onNewsClick;
			})){
			@Override public boolean onSystemEvent(SystemEvent event)
			{
			    if (event.getType() == SystemEvent.Type.REGULAR)
				switch(event.getCode())
				{
				case SAVE:
				return onAddLike();
				}
			    return super.onSystemEvent(event);
			}
		    };

		this.chatsArea = new ListArea<ConversationWithMessage>(listParams((params)->{
			    params.name = app.getStrings().conversationsAreaName();
			    params.model = new ListModel<>(app.chats);
			    params.appearance = new ChatsAppearance(app);
			    		}));
		
		//				this.requestsArea = new ListArea(listParams((params)->{}));
    final ActionInfo
    actionHomeWall = action("home-wall", "Стена", App.HOT_KEY_HOME_WALL, app.layouts()::homeWall),
    actionFriends = action("friends", "Друзья", App.HOT_KEY_FRIENDS, app.layouts()::friends),
    actionSuggestions = action("friendship-suggestions", "Вероятные знакомые", App.HOT_KEY_FRIENDSHIP_SUGGESTIONS, app.layouts()::friendshipSuggestions),
    actionPersonalInfo = action("personal-info", "Персональная информация", App.HOT_KEY_PERSONAL_INFO, app.layouts()::personalInfo);
		setAreaLayout(AreaLayout.LEFT_RIGHT,
			      newsArea, actions(actionFriends, actionHomeWall, actionSuggestions, actionPersonalInfo),
		  chatsArea, actions(actionFriends, actionHomeWall, actionSuggestions, actionPersonalInfo));
    }

    private boolean onNewsClick(ListArea<NewsfeedNewsfeedItemOneOf> area, int index, NewsfeedNewsfeedItemOneOf item)
    {
	final var post = item.getOneOf0();
	if (post.getSourceId() == null || post.getPostId() == null)
	    return false;
	final var taskId = app.newTaskId();
	return app.runTask(taskId, ()->{
		final var wallPost = app.getOperations().getWallPost(post.getSourceId().toString() + "_" + post.getPostId());
		final var likes = app.getOperations().getLikesWallPost(wallPost.getOwnerId(), wallPost.getId());
		app.finishedTask(taskId, ()->{
			final WallPostLayout layout = new WallPostLayout(app, wallPost, likes, ()->{
				app.setAreaLayout(this);
				setActiveArea(newsArea);
				return true;
			});
			app.setAreaLayout(layout);
			getLuwrain().announceActiveArea();
		    });
	    });
    }

        private boolean onAddLike()
    {
	final NewsfeedNewsfeedItemOneOf item = newsArea.selected();
	if (item == null)
	    return false;
	final var post = item.getOneOf0();
	if (post.getSourceId() == null || post.getPostId() == null)
	    return false;
	final var taskId = app.newTaskId();
	return app.runTask(taskId, ()->{
		final var wallPost = app.getOperations().getWallPost(post.getSourceId().toString() + "_" + post.getPostId());
		app.getOperations().addLikeWallPost(wallPost);
		app.finishedTask(taskId, ()->{
			getLuwrain().playSound(Sounds.OK);
		    });
	    });
    }

    
}
